using System;
using System.Collections.Generic;
using System.Linq;

namespace DeckOfCards.UnitTests
{
    public class Deck
    {
        public void Add(Card card)
        {
            if (IsUnique(card) == false)
            {
                throw new InvalidOperationException(
                    "Duplicate card.");
            }
            else
            {
                Cards.Add(card);
            }
        }

        private bool IsUnique(Card card)
        {
            var result = (from item in Cards
                          where
                          item.Rank == card.Rank &&
                          item.Suit == card.Suit
                          select item).FirstOrDefault();

            if (result == null)
            {
                return true;
            }
            else
            {
                return false;
            }
        }

        public int CardCount
        {
            get
            {
                return Cards.Count;
            }
        }

        private List<Card> m_Cards;
        public List<Card> Cards
        {
            get
            {
                if (m_Cards == null)
                {
                    m_Cards = new List<Card>();
                }

                return m_Cards;
            }
        }

        public void Initialize()
        {
            Initialize(Suit.Club);
            Initialize(Suit.Diamond);
            Initialize(Suit.Heart);
            Initialize(Suit.Spade);
        }

        private void Add(Suit suit, Rank rank)
        {
            Add(new Card(suit, rank));
        }

        private void Initialize(Suit suit)
        {
            Add(suit, Rank.Ace);
            Add(suit, Rank.Two);
            Add(suit, Rank.Three);
            Add(suit, Rank.Four);
            Add(suit, Rank.Five);
            Add(suit, Rank.Six);
            Add(suit, Rank.Seven);
            Add(suit, Rank.Eight);
            Add(suit, Rank.Nine);
            Add(suit, Rank.Ten);
            Add(suit, Rank.Jack);
            Add(suit, Rank.Queen);
            Add(suit, Rank.King);
        }
    }
}
