using System;
using System.Collections.Generic;
using System.Linq;
using Labs.Interfaces;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace Labs.Tests
{
public class MockCustomerRepository : ICustomerRepository
{
    public MockCustomerRepository()
    {

    }

    private bool m_WasIsAccountCodeInUseCalled = false;

    private List<string> m_IsAccountCodeInUseAttemptValues = 
        new List<string>();

    public bool IsAccountCodeInUse(string accountCode)
    {
        m_IsAccountCodeInUseAttemptValues.Add(accountCode);
        m_WasIsAccountCodeInUseCalled = true;

        if (NumberOfRequiredIsAccountCodeInUseAttempts == 0 ||
            m_IsAccountCodeInUseAttemptValues.Count >=
                 NumberOfRequiredIsAccountCodeInUseAttempts)
        {
            return false;
        }
        else
        {
            return true;
        }
    }

    public int NumberOfRequiredIsAccountCodeInUseAttempts { get; set; }

    public void AssertIsAccountCodeInUseWasCalled(int expectedCount)
    {
        Assert.AreEqual<int>(expectedCount,
            m_IsAccountCodeInUseAttemptValues.Count,
            "Method was not called the expected number of times.");
    }

    public void AssertReceivedAccountCodesWereDifferent()
    {
        CollectionAssert.AllItemsAreUnique(
            m_IsAccountCodeInUseAttemptValues,
            "IsAccountCodeInUse values should have been unique.");
    }

    private bool m_WasSaveCustomerCalled = false;

    public void SaveCustomer(string accountCode, string customerName)
    {
            m_WasSaveCustomerCalled = true;
        }

        public void AssertSaveCustomerWasCalled()
        {
            Assert.IsTrue(m_WasSaveCustomerCalled,
                "SaveCustomer() was not called.");
        }

        public void AssertIsAccountCodeInUseWasCalled()
        {
            Assert.IsTrue(m_WasIsAccountCodeInUseCalled,
                "IsAccountCodeInUse() was not called.");
        }
    }
}
