using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using DataAccessTesting.DataAccess;
using DataAccessTesting.Business;
using DataAccessTesting.Interfaces;

namespace DataAccessTesting.Tests
{
    [TestClass]
    public class SqlPersonRepositoryFixture
    {
        [TestMethod]
        public void SavedPersonDoesNotHaveZeroAsAnIdValue()
        {
            var instance = CreateAndSavePerson();

            Assert.AreNotEqual<int>(0, instance.Id,
                "Person Id value should not be zero.");
        }

        [TestInitialize]
        public void Initialize()
        {
            m_RepositoryInstance = null;
        }

        private IPersonRepository m_RepositoryInstance;
        public IPersonRepository RepositoryInstance
        {
            get
            {
                if (m_RepositoryInstance == null)
                {
                    m_RepositoryInstance = new SqlPersonRepository<Person>();
                }

                return m_RepositoryInstance;
            }
        }

        public IPerson CreateAndSavePerson()
        {
            IPerson instance = new Person();

            instance.FirstName = UnitTestUtility.GetUniqueString();
            instance.LastName = UnitTestUtility.GetUniqueString();
            instance.EmailAddress = UnitTestUtility.GetUniqueString();

            RepositoryInstance.Save(instance);

            return instance;
        }

        [TestMethod]
        public void GetPersonById()
        {
            var instance = CreateAndSavePerson();

            // reload the person 
            IPerson reloadedPerson =
                RepositoryInstance.GetById(instance.Id);

            AssertAreEqual(instance, reloadedPerson);
        }

        private void AssertAreEqual(IPerson expected, IPerson actual)
        {
            Assert.AreEqual<int>(expected.Id,
                actual.Id,
                "Id");
            Assert.AreEqual<string>(expected.FirstName,
                actual.FirstName,
                "FirstName");
            Assert.AreEqual<string>(expected.LastName,
                actual.LastName,
                "LastName");
            Assert.AreEqual<string>(expected.EmailAddress,
                actual.EmailAddress,
                "EmailAddress");
        }

        [TestMethod]
        public void ModifyPerson()
        {
            var instance = CreateAndSavePerson();

            // reload the person to get a copy
            IPerson clone =
                RepositoryInstance.GetById(instance.Id);

            // modify the original
            instance.FirstName = UnitTestUtility.GetUniqueString();
            instance.LastName = UnitTestUtility.GetUniqueString();
            instance.EmailAddress = UnitTestUtility.GetUniqueString();

            RepositoryInstance.Save(instance);

            Assert.AreEqual<int>(clone.Id,
                instance.Id,
                "Id value should not change after saving a modification.");

            AssertAreEqual(instance, RepositoryInstance.GetById(instance.Id));
        }

        [TestMethod]
        public void GetAllPersons()
        {
            // make sure there are at least 2 person records in the database
            CreateAndSavePerson();
            CreateAndSavePerson();

            var allPersons = RepositoryInstance.GetAll();

            Assert.IsNotNull(allPersons, "Got back a null result.");

            Assert.IsTrue(allPersons.Count >= 2,
                "Expected at least 2 person records but there were only '{0}'.",
                allPersons.Count);
        }
    }
}