﻿using System;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Labs.Business;

namespace Labs.Tests
{
    [TestClass]
    public class CustomerSearchPresenterFixture
    {
        [TestMethod]
        public void CheckInitialize()
        {
            ICustomerSearchView view = new CustomerSearchViewStub();

            var presenter = new CustomerSearchPresenter();

            presenter.Initialize(view);

            Assert.AreEqual<string>(String.Empty, view.SearchValue,
                "SearchValue was not empty.");
            Assert.IsNull(view.Results, "Results should be null.");
        }

        [TestMethod]
        public void SearchForUnknownValueReturnsNoResults()
        {
            ICustomerSearchView view = new CustomerSearchViewStub();

            var presenter = new CustomerSearchPresenter();

            presenter.Initialize(view);

            // search for something that doesn't exist
            view.SearchValue = Guid.NewGuid().ToString();
            presenter.Search(view);

            Assert.IsNotNull(view.Results, "Results was null.");
            Assert.AreEqual<int>(0, view.Results.Count,
                "Row count was wrong.");
        }

        [TestMethod]
        public void SearchForKnownRecordsReturnsResults()
        {
            ICustomerSearchView view = new CustomerSearchViewStub();

            var presenter = new CustomerSearchPresenter();

            presenter.Initialize(view);

            // search for known records
            view.SearchValue = "super";
            presenter.Search(view);

            Assert.IsNotNull(view.Results, "Results was null.");
            Assert.AreEqual<int>(2, view.Results.Count,
                "Row count was wrong.");
        }

        [TestMethod]
        public void SearchUsingEmptyStringReturnsNoResults()
        {
            ICustomerSearchView view = new CustomerSearchViewStub();

            var presenter = new CustomerSearchPresenter();

            presenter.Initialize(view);

            // empty search should return no records
            view.SearchValue = "";
            presenter.Search(view);

            Assert.IsNotNull(view.Results, "Results was null.");
            Assert.AreEqual<int>(0, view.Results.Count,
                "Row count was wrong.");
        }
    }
}
